package com.iden.bms.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.iden.common.entity.*;
import com.iden.common.enums.*;
import com.iden.common.service.*;
import com.iden.common.vo.*;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class WarningPersonService {

    @Resource
    private IdenPersonService idenPersonService;
    @Resource
    private IdenWarningPersonService idenWarningPersonService;

    @Resource
    private IdenPersonCrowdRefService idenPersonCrowdRefService;
    @Resource
    private IdenCrowdService idenCrowdService;


    /**
     * 根据小区查询预警人员列表分页
     * @return
     */
    public IPage<WarningPersonVO> listWarningPersonbyCommunity(String type, Long communityId, String nameOrCred, String populationType, Long crowdId, String beginDate, String endDate, UserLoginedConvertVO loginUser, PageReqVO pageReqVo) {
        QueryWrapper<IdenPerson> queryWrapper0 = new QueryWrapper<>();
        queryWrapper0.lambda().eq(StrUtil.isNotEmpty(populationType),IdenPerson::getPopulationType,populationType)
                .and(StrUtil.isNotEmpty(nameOrCred),wrapper -> wrapper.like(IdenPerson::getName,nameOrCred)
                .or().like(IdenPerson::getCredentialsCode,nameOrCred));
        if (crowdId != null) {
            queryWrapper0.apply(" EXISTS ( select dev_code from iden_person_crowd_ref ipcr where ipcr.person_id = iden_person.id  and ipcr.crowd_id  = " + crowdId + ")");
        }

        List<IdenPerson> listIdenPerson = idenPersonService.list(queryWrapper0);
        List<Long> listIdenPersonId = new ArrayList<>();
        if(CollUtil.isNotEmpty(listIdenPerson)){
            listIdenPerson.forEach(item->{
                listIdenPersonId.add(item.getId());
            });
        }

        IPage<IdenWarningPerson> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
        QueryWrapper<IdenWarningPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenWarningPerson::getType,type).eq(IdenWarningPerson::getCommunityId,communityId)
                .ge(StrUtil.isNotEmpty(beginDate),IdenWarningPerson::getDataDate,beginDate)
                .le(StrUtil.isNotEmpty(endDate),IdenWarningPerson::getDataDate,endDate)
                .in(CollUtil.isNotEmpty(listIdenPersonId),IdenWarningPerson::getPersonId,listIdenPersonId)
                .orderByAsc(IdenWarningPerson::getId);

        queryWrapper.groupBy("uid");
        queryWrapper.select(
                "max(community_id) communityId",
                "max(image) image",
                "sum(appear_frequency) appearFrequency",
                "max(is_continue) isContinue",
                "max(subdistrict) subdistrict",
                "max(stranger_type) strangerType",
                "max(handle_status) handleStatus",
                "max(remark) remark",
                "max(earliest_appear_time) earliestAppearTime",
                "max(continue_disappear_days) continueDisappearDays",
                "max(person_id) personId"
        );

        IPage<IdenWarningPerson> pageRes = this.idenWarningPersonService.page(page, queryWrapper);
        IPage<WarningPersonVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
        if(CollUtil.isNotEmpty(pageRes.getRecords())){
            List<WarningPersonVO> list = new ArrayList<>();
            pageRes.getRecords().forEach(item -> {
                WarningPersonVO resVO = new WarningPersonVO();
                BeanUtils.copyProperties(item,resVO);
                resVO.setTypeName(WarningTypeEnum.getValueToName(resVO.getType()));
                resVO.setStrangerTypeName(StrangerTypeEnum.getValueToName(resVO.getStrangerType()));
                resVO.setIsContinueName(IsContinueEnum.getValueToName(resVO.getIsContinue()));
                resVO.setHandleStatusName(HandleWarningStatusEnum.getValueToName(resVO.getHandleStatus()));
                Long personId = resVO.getPersonId();
                if (personId != null) {
                    IdenPerson idenPerson = this.idenPersonService.getById(personId);
                    PersonVO personVO = new PersonVO();
                    BeanUtils.copyProperties(idenPerson,personVO);
                    personVO.setPopulationTypeName(PopulationTypeEnum.getValueToName(personVO.getPopulationType()));
                    personVO.setTypeName(PersonTypeEnum.getValueToName(personVO.getType()));

                    QueryWrapper<IdenPersonCrowdRef> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.lambda().eq(IdenPersonCrowdRef::getPersonId,personVO.getId());
                    List<IdenPersonCrowdRef> listIdenPersonCrowdRef = idenPersonCrowdRefService.list(queryWrapper1);
                    StringBuilder sb = new StringBuilder();
                    if (CollUtil.isNotEmpty(listIdenPersonCrowdRef)) {
                        for(IdenPersonCrowdRef idenPersonCrowdRef : listIdenPersonCrowdRef){
                            IdenCrowd idenCrowd = idenCrowdService.getById(idenPersonCrowdRef.getCrowdId());
                            if(idenCrowd != null) {
                                sb.append(idenCrowd.getName()).append(",");
                            }
                        }
                    }
                    String crowdName = sb.toString();
                    if(crowdName != null && crowdName.endsWith(",")) {
                        crowdName = crowdName.substring(0, crowdName.length() - 1);
                    }
                    personVO.setCrowdName(crowdName);
                    resVO.setPersonVO(personVO);
                }
                list.add(resVO);
            });
            results.setRecords(list);
        }
        return results;
    }


}